package ch.phoenix.oss.quarkus.commons.tracing;

import io.smallrye.config.ConfigMapping;
import io.smallrye.config.WithConverter;
import io.smallrye.config.WithDefault;
import jakarta.ws.rs.core.HttpHeaders;
import java.util.Optional;
import java.util.Set;

@ConfigMapping(prefix = "phoenix.commons.tracing")
public interface TracingConfiguration {

    RequestFilterConfiguration requestFilter();

    interface RequestFilterConfiguration {

        Headers headers();

        interface Headers {

            Set<String> DEFAULT_REDACTED = Set.of(HttpHeaders.AUTHORIZATION.toLowerCase());

            /**
             * Optional set of headers to redact when tracing. By default, redacts
             * the 'Authorization' header.
             *
             * @return the set of headers to be redacted
             */
            Optional<Set<@WithConverter(LowerCaseStringConverter.class) String>> redact();
        }

        Path path();

        interface Path {

            @WithDefault("false")
            boolean includeRaw();
        }

        Query query();

        interface Query {

            Set<String> DEFAULT_REDACTED = Set.of("access_token", "refresh_token", "apikey");

            @WithDefault("false")
            boolean includeRaw();

            /**
             * Optional set of query params to redact when tracing. By default, redacts
             * the following params: 'access_token', 'refresh_token' and 'apikey'.
             *
             * @return the set of query params to be redacted
             */
            Optional<Set<@WithConverter(LowerCaseStringConverter.class) String>> redact();
        }
    }
}
