package ch.phoenix.oss.quarkus.commons.tracing;

import io.opentelemetry.api.trace.Span;
import io.quarkus.arc.DefaultBean;
import io.quarkus.logging.Log;
import jakarta.enterprise.context.ApplicationScoped;
import org.jboss.logging.MDC;

@DefaultBean
@ApplicationScoped
class TracingServiceImpl implements TracingService {

    private final Span span;

    TracingServiceImpl(Span span) {
        this.span = span;
    }

    @Override
    public void clearAll() {
        MDC.clear();
    }

    @Override
    public void trace(final String key, final Object value) {
        Log.tracef("tracing key=%s value=%s", key, value);
        MDC.put(key, value);
    }

    @Override
    public String getActor() {
        return (String) MDC.get(TracingConstants.ACTOR);
    }

    @Override
    public String getRequestPathRaw() {
        return (String) MDC.get(TracingConstants.REQUEST_PATH_RAW);
    }

    @Override
    public String getRequestMethod() {
        return (String) MDC.get(TracingConstants.REQUEST_METHOD);
    }

    @Override
    public String getRequestId() {
        return (String) MDC.get(TracingConstants.REQUEST_HEADERS + ".x-request-id");
    }

    @Override
    public String getTraceId() {
        return span.getSpanContext().getTraceId();
    }

    @Override
    public String getSpanId() {
        return span.getSpanContext().getSpanId();
    }

    @Override
    public String getClientIp() {
        return (String) MDC.get(TracingConstants.REQUEST_CLIENT_IP);
    }

    @Override
    public String getSchedulerJob() {
        return (String) MDC.get(TracingConstants.SCHEDULER_JOB_NAME);
    }
}
